@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -38,18 +38,37 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)fio.c	2.23 (gritter) 11/8/02";
+static char sccsid[] = "@(#)fio.c	2.68 (gritter) 11/6/04";
 #endif
 #endif /* not lint */
 
 #include "rcv.h"
+#include <sys/stat.h>
 #include <sys/file.h>
-#ifdef	HAVE_SYS_WAIT_H
 #include <sys/wait.h>
-#endif
-#ifdef	HAVE_WORDEXP_H
+#ifdef	HAVE_WORDEXP
 #include <wordexp.h>
-#endif
+#endif	/* HAVE_WORDEXP */
+#include <unistd.h>
+
+#if defined (USE_NSS)
+#include <nss.h>
+#include <ssl.h>
+#elif defined (USE_OPENSSL)
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#include <openssl/x509v3.h>
+#include <openssl/x509.h>
+#include <openssl/rand.h>
+#endif	/* USE_SSL */
+#ifdef	HAVE_SOCKETS
+#include <sys/socket.h>
+#include <netdb.h>
+#include <netinet/in.h>
+#ifdef	HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif	/* HAVE_ARPA_INET_H */
+#endif	/* HAVE_SOCKETS */
 
 #include <errno.h>
 #include "extern.h"
@@ -59,21 +78,20 @@ static char sccsid[] = "@(#)fio.c	2.23 (gritter) 11/8/02";
  *
  * File I/O.
  */
-static void	makemessage __P((void));
-static void	append __P((struct message *));
-static char	*globname __P((char *));
-static size_t	length_of_line __P((const char *, size_t));
-static char	*fgetline_byone __P((char **, size_t *, size_t *,
-			FILE *, int, size_t));
-static enum okay	get_header __P((struct message *));
+
+static void makemessage(void);
+static void append(struct message *mp);
+static char *globname(char *name);
+static size_t length_of_line(const char *line, size_t linesize);
+static char *fgetline_byone(char **line, size_t *linesize, size_t *llen,
+		FILE *fp, int appendnl, size_t n);
+static enum okay get_header(struct message *mp);
 
 /*
  * Set up the input pointers while copying the mail file into /tmp.
  */
 void
-setptr(ibuf, offset)
-	FILE *ibuf;
-	off_t offset;
+setptr(FILE *ibuf, off_t offset)
 {
 	int c;
 	size_t count;
@@ -87,11 +105,8 @@ setptr(ibuf, offset)
 	maybe = 1;
 	inhead = 0;
 	thiscnt = 0;
-	this.m_flag = MUSED|MNEW;
-	this.m_size = 0;
-	this.m_lines = 0;
-	this.m_block = 0;
-	this.m_offset = 0;
+	memset(&this, 0, sizeof this);
+	this.m_flag = MUSED|MNEW|MNEWEST;
 	filesize = mailsize - offset;
 	for (;;) {
 		if (fgetline(&linebuf, &linesize, &filesize, &count, ibuf, 0)
@@ -110,7 +125,7 @@ setptr(ibuf, offset)
 		if (linebuf[0] == '\0')
 			linebuf[0] = '.';
 #endif
-		(void) fwrite(linebuf, sizeof *linebuf, count, mb.mb_otf);
+		fwrite(linebuf, sizeof *linebuf, count, mb.mb_otf);
 		if (ferror(mb.mb_otf)) {
 			perror("/tmp");
 			exit(1);
@@ -123,8 +138,8 @@ setptr(ibuf, offset)
 			this.m_have = HAVE_HEADER|HAVE_BODY;
 			if (thiscnt++ > 0)
 				append(&this);
-			msgcount++;
-			this.m_flag = MUSED|MNEW;
+			msgCount++;
+			this.m_flag = MUSED|MNEW|MNEWEST;
 			this.m_size = 0;
 			this.m_lines = 0;
 			this.m_block = nail_blockof(offset);
@@ -143,10 +158,26 @@ setptr(ibuf, offset)
 							this.m_flag |= MREAD;
 						else if (c == 'O')
 							this.m_flag &= ~MNEW;
-					inhead = 0;
 					break;
 				}
-				if (*cp != c && *cp != toupper(c))
+				if (*cp != c && *cp != upperconv(c))
+					break;
+			}
+			for (cp = linebuf, cp2 = "x-status";; cp++) {
+				if ((c = *cp2++) == 0) {
+					while (c = *cp++, whitechar(c));
+					if (cp[-1] != ':')
+						break;
+					while ((c = *cp++) != '\0')
+						if (c == 'F')
+							this.m_flag |= MFLAGGED;
+						else if (c == 'A')
+							this.m_flag|=MANSWERED;
+						else if (c == 'T')
+							this.m_flag|=MDRAFTED;
+					break;
+				}
+				if (*cp != c && *cp != upperconv(c))
 					break;
 			}
 		}
@@ -165,13 +196,10 @@ setptr(ibuf, offset)
  * characters written, including the newline.
  */
 int
-putline(obuf, linebuf, count)
-	FILE *obuf;
-	char *linebuf;
-	size_t count;
+putline(FILE *obuf, char *linebuf, size_t count)
 {
-	(void) fwrite(linebuf, sizeof *linebuf, count, obuf);
-	(void) sputc('\n', obuf);
+	fwrite(linebuf, sizeof *linebuf, count, obuf);
+	putc('\n', obuf);
 	if (ferror(obuf))
 		return (-1);
 	return (count + 1);
@@ -185,13 +213,9 @@ putline(obuf, linebuf, count)
  * n is the number of characters already read.
  */
 int
-readline_restart(ibuf, linebuf, linesize, n)
-	FILE *ibuf;
-	char **linebuf;
-	size_t *linesize;
-	size_t n;
+readline_restart(FILE *ibuf, char **linebuf, size_t *linesize, size_t n)
 {
-	ssize_t sz;
+	long sz;
 
 	clearerr(ibuf);
 	/*
@@ -247,45 +271,47 @@ again:
  * passed message pointer.
  */
 FILE *
-setinput(mp, need)
-	struct message *mp;
-	enum needspec need;
+setinput(struct mailbox *mp, struct message *m, enum needspec need)
 {
 	enum okay ok = STOP;
 
 	switch (need) {
 	case NEED_HEADER:
-		if (mp->m_have & HAVE_HEADER)
+		if (m->m_have & HAVE_HEADER)
 			ok = OKAY;
 		else
-			ok = get_header(mp);
+			ok = get_header(m);
 		break;
 	case NEED_BODY:
-		if (mp->m_have & HAVE_BODY)
+		if (m->m_have & HAVE_BODY)
 			ok = OKAY;
 		else
-			ok = get_body(mp);
+			ok = get_body(m);
+		break;
+	case NEED_UNSPEC:
+		ok = OKAY;
 		break;
 	}
 	if (ok != OKAY)
 		return NULL;
-	fflush(mb.mb_otf);
-	if (fseek(mb.mb_itf, (long)nail_positionof(mp->m_block,
-					mp->m_offset), 0) < 0) {
+	fflush(mp->mb_otf);
+	if (fseek(mp->mb_itf, (long)nail_positionof(m->m_block,
+					m->m_offset), SEEK_SET) < 0) {
 		perror("fseek");
 		panic(catgets(catd, CATSET, 77, "temporary file seek"));
 	}
-	return (mb.mb_itf);
+	return (mp->mb_itf);
 }
 
 struct message *
-setdot(mp)
-	struct message *mp;
+setdot(struct message *mp)
 {
-	if (dot != mp)
+	if (dot != mp) {
 		prevdot = dot;
+		did_print_dot = 0;
+	}
 	dot = mp;
-	did_print_dot = 0;
+	uncollapse1(dot, 0);
 	return dot;
 }
 
@@ -294,34 +320,32 @@ setdot(mp)
  * a dynamically allocated message structure.
  */
 static void
-makemessage()
+makemessage(void)
 {
-	if (msgcount == 0)
+	if (msgCount == 0)
 		append(NULL);
 	setdot(message);
-	message[msgcount].m_size = 0;
-	message[msgcount].m_lines = 0;
+	message[msgCount].m_size = 0;
+	message[msgCount].m_lines = 0;
 }
 
 /*
  * Append the passed message descriptor onto the message structure.
  */
 static void
-append(mp)
-	struct message *mp;
+append(struct message *mp)
 {
-	if (msgcount + 1 >= msgspace)
+	if (msgCount + 1 >= msgspace)
 		message = srealloc(message, (msgspace += 64) * sizeof *message);
-	if (msgcount > 0)
-		message[msgcount - 1] = *mp;
+	if (msgCount > 0)
+		message[msgCount - 1] = *mp;
 }
 
 /*
  * Delete a file, but only if the file is a plain file.
  */
 int
-rm(name)
-	char *name;
+rm(char *name)
 {
 	struct stat sb;
 
@@ -340,7 +364,7 @@ static sigset_t nset, oset;
  * Hold signals SIGHUP, SIGINT, and SIGQUIT.
  */
 void
-holdsigs()
+holdsigs(void)
 {
 
 	if (sigdepth++ == 0) {
@@ -356,7 +380,7 @@ holdsigs()
  * Release signals SIGHUP, SIGINT, and SIGQUIT.
  */
 void
-relsesigs()
+relsesigs(void)
 {
 
 	if (--sigdepth == 0)
@@ -368,8 +392,7 @@ relsesigs()
  * the passed buffer.
  */
 off_t
-fsize(iob)
-	FILE *iob;
+fsize(FILE *iob)
 {
 	struct stat sbuf;
 
@@ -390,8 +413,7 @@ fsize(iob)
  * Return the file name as a dynamic string.
  */
 char *
-expand(name)
-	char *name;
+expand(char *name)
 {
 	char xname[PATHSIZE];
 	char foldbuf[PATHSIZE];
@@ -428,10 +450,21 @@ next:
 			name = "~/mbox";
 		/* fall through */
 	}
-	if (name[0] == '+' && getfold(foldbuf, sizeof foldbuf) >= 0) {
-		snprintf(xname, sizeof xname, "%s/%s", foldbuf, name + 1);
+	if (name[0] == '@' && which_protocol(mailname) == PROTO_IMAP) {
+		snprintf(xname, sizeof xname, "%s/%s", protbase(mailname),
+				&name[1]);
 		name = savestr(xname);
 	}
+	if (name[0] == '+' && getfold(foldbuf, sizeof foldbuf) >= 0) {
+		if (which_protocol(foldbuf) == PROTO_IMAP &&
+				strcmp(foldbuf, protbase(foldbuf)))
+		snprintf(xname, sizeof xname, "%s%s", foldbuf, name+1);
+		else
+			snprintf(xname, sizeof xname, "%s/%s", foldbuf, name+1);
+		name = savestr(xname);
+		if (foldbuf[0] == '%' && foldbuf[1] == ':')
+			goto next;
+	}
 	/* catch the most common shell meta character */
 	if (name[0] == '~' && (name[1] == '/' || name[1] == '\0')) {
 		snprintf(xname, sizeof xname, "%s%s", homedir, name + 1);
@@ -443,8 +476,7 @@ next:
 }
 
 static char *
-globname(name)
-	char *name;
+globname(char *name)
 {
 #ifdef	HAVE_WORDEXP
 	wordexp_t we;
@@ -509,7 +541,7 @@ globname(name)
 	}
 	snprintf(cmdbuf, sizeof cmdbuf, "echo %s", name);
 	if ((shell = value("SHELL")) == NULL)
-		shell = PATH_CSHELL;
+		shell = SHELL;
 	pid = start_command(shell, 0, -1, pivec[1], "-c", cmdbuf, NULL);
 	if (pid < 0) {
 		close(pivec[0]);
@@ -559,15 +591,15 @@ again:
  * Determine the current folder directory name.
  */
 int
-getfold(name, size)
-	char *name;
-	int size;
+getfold(char *name, int size)
 {
 	char *folder;
+	enum protocol	p;
 
 	if ((folder = value("folder")) == NULL)
 		return (-1);
-	if (*folder == '/') {
+	if (*folder == '/' || (p = which_protocol(folder)) != PROTO_FILE &&
+			p != PROTO_MAILDIR) {
 		strncpy(name, folder, size);
 		name[size-1]='\0';
 	} else {
@@ -580,7 +612,7 @@ getfold(name, size)
  * Return the name of the dead.letter file.
  */
 char *
-getdeadletter()
+getdeadletter(void)
 {
 	char *cp;
 
@@ -604,9 +636,7 @@ getdeadletter()
  * newline or the last character read.
  */
 static size_t
-length_of_line(line, linesize)
-const char *line;
-size_t linesize;
+length_of_line(const char *line, size_t linesize)
 {
 	register size_t i;
 
@@ -631,12 +661,10 @@ size_t linesize;
  * appendnl - always terminate line with \n, append if necessary.
  */
 char *
-fgetline(line, linesize, count, llen, fp, appendnl)
-char **line;
-size_t *linesize, *count, *llen;
-FILE *fp;
+fgetline(char **line, size_t *linesize, size_t *count, size_t *llen,
+		FILE *fp, int appendnl)
 {
-	ssize_t i_llen, sz;
+	long i_llen, sz;
 
 	if (count == NULL)
 		/*
@@ -681,11 +709,8 @@ FILE *fp;
  * Read a line, one character at once.
  */
 static char *
-fgetline_byone(line, linesize, llen, fp, appendnl, n)
-char **line;
-size_t *linesize, *llen;
-FILE *fp;
-size_t n;
+fgetline_byone(char **line, size_t *linesize, size_t *llen,
+		FILE *fp, int appendnl, size_t n)
 {
 	int c;
 
@@ -694,7 +719,7 @@ size_t n;
 	for (;;) {
 		if (n >= *linesize - 128)
 			*line = srealloc(*line, *linesize += 256);
-		c = sgetc(fp);
+		c = getc(fp);
 		if (c != EOF) {
 			(*line)[n++] = c;
 			(*line)[n] = '\0';
@@ -718,7 +743,7 @@ size_t n;
 }
 
 void
-newline_appended()
+newline_appended(void)
 {
 	fprintf(stderr, catgets(catd, CATSET, 208,
 			"warning: incomplete line - newline appended\n"));
@@ -726,14 +751,17 @@ newline_appended()
 }
 
 static enum okay
-get_header(mp)
-	struct message *mp;
+get_header(struct message *mp)
 {
 	switch (mb.mb_type) {
 	case MB_FILE:
+	case MB_MAILDIR:
 		return OKAY;
 	case MB_POP3:
 		return pop3_header(mp);
+	case MB_IMAP:
+	case MB_CACHE:
+		return imap_header(mp);
 	case MB_VOID:
 		return STOP;
 	}
@@ -742,14 +770,17 @@ get_header(mp)
 }
 
 enum okay
-get_body(mp)
-	struct message *mp;
+get_body(struct message *mp)
 {
 	switch (mb.mb_type) {
 	case MB_FILE:
+	case MB_MAILDIR:
 		return OKAY;
 	case MB_POP3:
 		return pop3_body(mp);
+	case MB_IMAP:
+	case MB_CACHE:
+		return imap_body(mp);
 	case MB_VOID:
 		return STOP;
 	}
@@ -757,34 +788,357 @@ get_body(mp)
 	return STOP;
 }
 
-/*
- * This is fputs with conversion to \r\n format.
- */
-int
-crlfputs(s, stream)
-char *s;
-FILE *stream;
-{
-	size_t len, wsz = 0;
+#ifdef	HAVE_SOCKETS
+static long xwrite(int fd, const char *data, size_t sz);
 
-	if ((len = strlen(s)) == 0)
-		return 0;
-	if (s[len - 1] == '\n' && (len == 1 || s[len - 2] != '\r')) {
-		if (--len > 0)
-			wsz = fwrite(s, sizeof *s, len, stream);
-		if (wsz > 0 || len == 0) {
-			if (fputc('\r', stream) != EOF) {
-				wsz++;
-				if (fputc('\n', stream) != EOF)
-					wsz++;
-				else
-					wsz = 0;
+static long
+xwrite(int fd, const char *data, size_t sz)
+{
+	long wo, wt = 0;
+
+	do {
+		if ((wo = write(fd, data + wt, sz - wt)) < 0) {
+			if (errno == EINTR)
+				continue;
+			else
+				return -1;
+		}
+		wt += wo;
+	} while (wt < sz);
+	return sz;
+}
+
+int
+sclose(struct sock *sp)
+{
+	int	i;
+
+	if (sp->s_fd > 0) {
+		if (sp->s_onclose != NULL)
+			(*sp->s_onclose)();
+#if defined (USE_NSS)
+		if (sp->s_use_ssl) {
+			sp->s_use_ssl = 0;
+			i = PR_Close(sp->s_prfd) == PR_SUCCESS ? 0 : -1;
+			sp->s_prfd = NULL;
+		} else
+#elif defined (USE_OPENSSL)
+		if (sp->s_use_ssl) {
+			sp->s_use_ssl = 0;
+			SSL_shutdown(sp->s_ssl);
+			SSL_free(sp->s_ssl);
+			sp->s_ssl = NULL;
+			SSL_CTX_free(sp->s_ctx);
+			sp->s_ctx = NULL;
+		}
+#endif	/* USE_SSL */
+		{
+			i = close(sp->s_fd);
+		}
+		sp->s_fd = -1;
+		return i;
+	}
+	sp->s_fd = -1;
+	return 0;
+}
+
+enum okay
+swrite(struct sock *sp, const char *data)
+{
+	return swrite1(sp, data, strlen(data), 0);
+}
+
+enum okay
+swrite1(struct sock *sp, const char *data, int sz, int use_buffer)
+{
+	int	x;
+
+	if (use_buffer > 0) {
+		int	di;
+		enum okay	ok;
+
+		if (sp->s_wbuf == NULL) {
+			sp->s_wbufsize = 4096;
+			sp->s_wbuf = smalloc(sp->s_wbufsize);
+			sp->s_wbufpos = 0;
+		}
+		while (sp->s_wbufpos + sz > sp->s_wbufsize) {
+			di = sp->s_wbufsize - sp->s_wbufpos;
+			sz -= di;
+			if (sp->s_wbufpos > 0) {
+				memcpy(&sp->s_wbuf[sp->s_wbufpos], data, di);
+				ok = swrite1(sp, sp->s_wbuf,
+						sp->s_wbufsize, -1);
 			} else
-				wsz = 0;
+				ok = swrite1(sp, data,
+						sp->s_wbufsize, -1);
+			if (ok != OKAY)
+				return STOP;
+			data += di;
+			sp->s_wbufpos = 0;
+		}
+		if (sz == sp->s_wbufsize) {
+			ok = swrite1(sp, data, sp->s_wbufsize, -1);
+			if (ok != OKAY)
+				return STOP;
+		} else if (sz) {
+			memcpy(&sp->s_wbuf[sp->s_wbufpos], data, sz);
+			sp->s_wbufpos += sz;
+		}
+		return OKAY;
+	} else if (use_buffer == 0 && sp->s_wbuf != NULL &&
+			sp->s_wbufpos > 0) {
+		x = sp->s_wbufpos;
+		sp->s_wbufpos = 0;
+		if (swrite1(sp, sp->s_wbuf, x, -1) != OKAY)
+			return STOP;
+	}
+	if (sz == 0)
+		return OKAY;
+#if defined (USE_NSS)
+	if (sp->s_use_ssl) {
+		x = PR_Write(sp->s_prfd, data, sz);
+	} else
+#elif defined (USE_OPENSSL)
+	if (sp->s_use_ssl) {
+ssl_retry:	x = SSL_write(sp->s_ssl, data, sz);
+		if (x < 0) {
+			switch (SSL_get_error(sp->s_ssl, x)) {
+			case SSL_ERROR_WANT_READ:
+			case SSL_ERROR_WANT_WRITE:
+				goto ssl_retry;
+			}
 		}
 	} else
-		wsz = fwrite(s, sizeof *s, len, stream);
-	if (wsz == 0)
-		return EOF;
-	return wsz;
+#endif	/* USE_SSL */
+	{
+		x = xwrite(sp->s_fd, data, sz);
+	}
+	if (x != sz) {
+		char	o[512];
+		snprintf(o, sizeof o, "%s write error",
+				sp->s_desc ? sp->s_desc : "socket");
+#if defined (USE_NSS)
+		sp->s_use_ssl ? nss_gen_err(o) : perror(o);
+#elif defined (USE_OPENSSL)
+		sp->s_use_ssl ? ssl_gen_err(o) : perror(o);
+#else	/* !USE_SSL */
+		perror(o);
+#endif	/* !USE_SSL */
+		if (x < 0)
+			sclose(sp);
+		return STOP;
+	}
+	return OKAY;
 }
+
+int
+sgetline(char **line, size_t *linesize, size_t *linelen, struct sock *sp)
+{
+	char	*lp = *line;
+
+	if (sp->s_rsz < 0) {
+		sclose(sp);
+		return sp->s_rsz;
+	}
+	do {
+		if (*line == NULL || lp > &(*line)[*linesize - 128]) {
+			size_t diff = lp - *line;
+			*line = srealloc(*line, *linesize += 256);
+			lp = &(*line)[diff];
+		}
+		if (sp->s_rbufptr == NULL ||
+				sp->s_rbufptr >= &sp->s_rbuf[sp->s_rsz]) {
+#if defined (USE_NSS)
+			if (sp->s_use_ssl) {
+				if ((sp->s_rsz = PR_Read(sp->s_prfd,
+						sp->s_rbuf,
+						sizeof sp->s_rbuf)) <= 0) {
+					if (sp->s_rsz < 0) {
+						char	o[512];
+						snprintf(o, sizeof o,
+							sp->s_desc ?
+								sp->s_desc :
+								"socket");
+						nss_gen_err(o);
+					}
+					break;
+				}
+			} else
+#elif defined (USE_OPENSSL)
+			if (sp->s_use_ssl) {
+		ssl_retry:	if ((sp->s_rsz = SSL_read(sp->s_ssl,
+						sp->s_rbuf,
+						sizeof sp->s_rbuf)) <= 0) {
+					if (sp->s_rsz < 0) {
+						char	o[512];
+						switch(SSL_get_error(sp->s_ssl,
+							sp->s_rsz)) {
+						case SSL_ERROR_WANT_READ:
+						case SSL_ERROR_WANT_WRITE:
+							goto ssl_retry;
+						}
+						snprintf(o, sizeof o,
+							sp->s_desc ?
+								sp->s_desc :
+								"socket");
+						ssl_gen_err(o);
+
+					}
+					break;
+				}
+			} else
+#endif	/* USE_SSL */
+			{
+			again:	if ((sp->s_rsz = read(sp->s_fd, sp->s_rbuf,
+						sizeof sp->s_rbuf)) <= 0) {
+					if (sp->s_rsz < 0) {
+						char	o[512];
+						if (errno == EINTR)
+							goto again;
+						snprintf(o, sizeof o,
+							sp->s_desc ?
+								sp->s_desc :
+								"socket");
+						perror(o);
+					}
+					break;
+				}
+			}
+			sp->s_rbufptr = sp->s_rbuf;
+		}
+	} while ((*lp++ = *sp->s_rbufptr++) != '\n');
+	*lp = '\0';
+	if (linelen)
+		*linelen = lp - *line;
+	return lp - *line;
+}
+
+enum okay
+sopen(const char *xserver, struct sock *sp, int use_ssl,
+		const char *uhp, const char *portstr, int verbose)
+{
+#ifdef	HAVE_IPv6_FUNCS
+	char	hbuf[NI_MAXHOST];
+	struct addrinfo	hints, *res0, *res;
+#else	/* !HAVE_IPv6_FUNCS */
+	struct sockaddr_in	servaddr;
+	struct in_addr	**pptr;
+	struct hostent	*hp;
+	struct servent	*ep;
+	unsigned short	port = 0;
+#endif	/* !HAVE_IPv6_FUNCS */
+	int	sockfd;
+	char	*cp;
+	char	*server = (char *)xserver;
+
+	if ((cp = strchr(server, ':')) != NULL) {
+		portstr = &cp[1];
+#ifndef	HAVE_IPv6_FUNCS
+		port = strtol(portstr, NULL, 10);
+#endif	/* HAVE_IPv6_FUNCS */
+		server = salloc(cp - xserver + 1);
+		memcpy(server, xserver, cp - xserver);
+		server[cp - xserver] = '\0';
+	}
+#ifdef	HAVE_IPv6_FUNCS
+	memset(&hints, 0, sizeof hints);
+	hints.ai_socktype = SOCK_STREAM;
+	if (verbose)
+		fprintf(stderr, "Resolving host %s . . .", server);
+	if (getaddrinfo(server, portstr, &hints, &res0) != 0) {
+		fprintf(stderr, catgets(catd, CATSET, 252,
+				"Could not resolve host: %s\n"), server);
+		return STOP;
+	} else if (verbose)
+		fprintf(stderr, " done.\n");
+	sockfd = -1;
+	for (res = res0; res != NULL && sockfd < 0; res = res->ai_next) {
+		if (verbose) {
+			if (getnameinfo(res->ai_addr, res->ai_addrlen,
+						hbuf, sizeof hbuf, NULL, 0,
+						NI_NUMERICHOST) != 0)
+				strcpy(hbuf, "unknown host");
+			fprintf(stderr, catgets(catd, CATSET, 192,
+					"Connecting to %s . . ."), hbuf);
+		}
+		if ((sockfd = socket(res->ai_family, res->ai_socktype,
+				res->ai_protocol)) >= 0) {
+			if (connect(sockfd, res->ai_addr, res->ai_addrlen)!=0) {
+				close(sockfd);
+				sockfd = -1;
+			}
+		}
+	}
+	if (sockfd < 0) {
+		perror(catgets(catd, CATSET, 254, "could not connect"));
+		freeaddrinfo(res0);
+		return STOP;
+	}
+	freeaddrinfo(res0);
+#else	/* !HAVE_IPv6_FUNCS */
+	if (port == 0) {
+		if ((ep = getservbyname((char *)portstr, "tcp")) == NULL) {
+			if (equal(portstr, "smtp"))
+				port = htons(25);
+			else if (equal(portstr, "smtps"))
+				port = htons(465);
+			else if (equal(portstr, "imap"))
+				port = htons(143);
+			else if (equal(portstr, "imaps"))
+				port = htons(993);
+			else if (equal(portstr, "pop3"))
+				port = htons(110);
+			else if (equal(portstr, "pop3s"))
+				port = htons(995);
+			else {
+				fprintf(stderr, catgets(catd, CATSET, 251,
+					"Unknown service: %s\n"), portstr);
+				return STOP;
+			}
+		} else
+			port = ep->s_port;
+	} else
+		port = htons(port);
+	if (verbose)
+		fprintf(stderr, "Resolving host %s . . .", server);
+	if ((hp = gethostbyname(server)) == NULL) {
+		fprintf(stderr, catgets(catd, CATSET, 252,
+				"Could not resolve host: %s\n"), server);
+		return STOP;
+	} else if (verbose)
+		fprintf(stderr, " done.\n");
+	pptr = (struct in_addr **)hp->h_addr_list;
+	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
+		perror(catgets(catd, CATSET, 253, "could not create socket"));
+		return STOP;
+	}
+	memset(&servaddr, 0, sizeof servaddr);
+	servaddr.sin_family = AF_INET;
+	servaddr.sin_port = port;
+	memcpy(&servaddr.sin_addr, *pptr, sizeof(struct in_addr));
+	if (verbose)
+		fprintf(stderr, catgets(catd, CATSET, 192,
+				"Connecting to %s . . ."), inet_ntoa(**pptr));
+	if (connect(sockfd, (struct sockaddr *)&servaddr, sizeof servaddr)
+			!= 0) {
+		perror(catgets(catd, CATSET, 254, "could not connect"));
+		return STOP;
+	}
+#endif	/* !HAVE_IPv6_FUNCS */
+	if (verbose)
+		fputs(catgets(catd, CATSET, 193, " connected.\n"), stderr);
+	memset(sp, 0, sizeof *sp);
+	sp->s_fd = sockfd;
+#if defined (USE_SSL)
+	if (use_ssl) {
+		enum okay ok;
+
+		if ((ok = ssl_open(server, sp, uhp)) != OKAY)
+			sclose(sp);
+		return ok;
+	}
+#endif	/* USE_SSL */
+	return OKAY;
+}
+#endif	/* HAVE_SOCKETS */
