@@ -40,7 +40,7 @@
 #ifdef	DOSCCS
 static char copyright[]
 = "@(#) Copyright (c) 2000, 2002 Gunnar Ritter. All rights reserved.\n";
-static char sccsid[]  = "@(#)mime.c	2.51 (gritter) 12/25/04";
+static char sccsid[]  = "@(#)mime.c	2.59 (gritter) 6/9/05";
 #endif /* DOSCCS */
 #endif /* not lint */
 
@@ -245,7 +245,7 @@ checkaddrs(struct name *np)
 	return np;
 }
 
-static char defcharset[] = "iso-8859-1";
+static char defcharset[] = "utf-8";
 
 /*
  * Get the character set dependant on the conversion.
@@ -255,8 +255,10 @@ getcharset(int isclean)
 {
 	char *charset;
 
-	if (isclean & MIME_HIGHBIT) {
-		charset = value("charset");
+	if (isclean & (MIME_CTRLCHAR|MIME_HASNUL))
+		charset = NULL;
+	else if (isclean & MIME_HIGHBIT) {
+		charset = wantcharset ? wantcharset : value("charset");
 		if (charset == NULL) {
 			charset = defcharset;
 		}
@@ -299,43 +301,46 @@ has_highbit(const char *s)
 	return 0;
 }
 
+static int
+name_highbit(struct name *np)
+{
+	while (np) {
+		if (has_highbit(np->n_name) || has_highbit(np->n_fullname))
+			return 1;
+		np = np->n_flink;
+	}
+	return 0;
+}
+
 char *
 need_hdrconv(struct header *hp, enum gfield w)
 {
-	struct name *np;
-
 	if (w & GIDENT) {
-		if (has_highbit(myaddr()))
+		if (hp->h_from && name_highbit(hp->h_from))
 			goto needs;
-		if (has_highbit(value("ORGANIZATION")))
+		else if (has_highbit(myaddrs()))
 			goto needs;
-		if (has_highbit(value("replyto")))
+		if (hp->h_organization && has_highbit(hp->h_organization))
 			goto needs;
-	}
-	if (w & GTO) {
-		for (np = hp->h_to; np; np = np->n_flink)
-			if (has_highbit(np->n_name))
-				goto needs;
-	}
-	if (w & GCC) {
-		for (np = hp->h_cc; np; np = np->n_flink)
-			if (has_highbit(np->n_name))
-				goto needs;
-	}
-	if (w & GBCC) {
-		for (np = hp->h_bcc; np; np = np->n_flink)
-			if (has_highbit(np->n_name))
-				goto needs;
-	}
-	if (w & GREPLYTO) {
-		for (np = hp->h_replyto; np; np = np->n_flink)
-			if (has_highbit(np->n_name))
-				goto needs;
-	}
-	if (w & GSUBJECT) {
-		if (has_highbit(hp->h_subject))
+		else if (has_highbit(value("ORGANIZATION")))
+			goto needs;
+		if (hp->h_replyto && name_highbit(hp->h_replyto))
+			goto needs;
+		else if (has_highbit(value("replyto")))
+			goto needs;
+		if (hp->h_sender && name_highbit(hp->h_sender))
+			goto needs;
+		else if (has_highbit(value("sender")))
 			goto needs;
 	}
+	if (w & GTO && name_highbit(hp->h_to))
+		goto needs;
+	if (w & GCC && name_highbit(hp->h_cc))
+		goto needs;
+	if (w & GBCC && name_highbit(hp->h_bcc))
+		goto needs;
+	if (w & GSUBJECT && has_highbit(hp->h_subject))
+		goto needs;
 	return NULL;
 needs:	return getcharset(MIME_HIGHBIT);
 }
@@ -758,15 +763,15 @@ gettextconversion(void)
 	int convert;
 
 	if ((p = value("encoding")) == NULL)
-		return CONV_NONE;
+		return CONV_8BIT;
 	if (equal(p, "quoted-printable"))
 		convert = CONV_TOQP;
 	else if (equal(p, "8bit"))
-		convert = CONV_NONE;
+		convert = CONV_8BIT;
 	else {
 		fprintf(stderr, catgets(catd, CATSET, 177,
 			"Warning: invalid encoding %s, using 8bit\n"), p);
-		convert = CONV_NONE;
+		convert = CONV_8BIT;
 	}
 	return convert;
 }
@@ -1529,8 +1534,9 @@ mime_write(void *ptr, size_t size, size_t nmemb, FILE *f,
 #ifdef	HAVE_ICONV
 	if (csize < sizeof mptr && (dflags & TD_ICONV)
 			&& iconvd != (iconv_t)-1
-			&& (convert == CONV_TOQP || convert == CONV_TOB64
-				|| convert == CONV_TOHDR)) {
+			&& (convert == CONV_TOQP || convert == CONV_8BIT ||
+				convert == CONV_TOB64 ||
+				convert == CONV_TOHDR)) {
 		inleft = csize;
 		outleft = sizeof mptr;
 		nptr = mptr;
@@ -1561,6 +1567,10 @@ mime_write(void *ptr, size_t size, size_t nmemb, FILE *f,
 	case CONV_TOQP:
 		sz = mime_write_toqp(&in, f, mustquote_body);
 		break;
+	case CONV_8BIT:
+		sz = prefixwrite(in.s, sizeof *in.s, in.l, f,
+				prefix, prefixlen);
+		break;
 	case CONV_FROMB64_T:
 		is_text = 1;
 		/*FALLTHROUGH*/
