@@ -1,7 +1,7 @@
 /*
  * Nail - a mail user agent derived from Berkeley Mail.
  *
- * Copyright (c) 2000-2002 Gunnar Ritter, Freiburg i. Br., Germany.
+ * Copyright (c) 2000-2004 Gunnar Ritter, Freiburg i. Br., Germany.
  */
 /*
  * Copyright (c) 1980, 1993
@@ -38,32 +38,34 @@
 
 #ifndef lint
 #ifdef	DOSCCS
-static char sccsid[] = "@(#)quit.c	2.7 (gritter) 3/30/03";
+static char sccsid[] = "@(#)quit.c	2.27 (gritter) 11/3/04";
 #endif
 #endif /* not lint */
 
 #include "rcv.h"
+#include "extern.h"
 #include <stdio.h>
 #include <errno.h>
-#include "extern.h"
-#ifdef	HAVE_SYS_FILE_H
 #include <sys/file.h>
-#endif
+#include <sys/stat.h>
+#include <unistd.h>
+#include <fcntl.h>
 
 /*
  * Rcv -- receive mail rationally.
  *
  * Termination processing.
  */
-static void	edstop __P((void));
+
+static int writeback(FILE *res, FILE *obuf);
+static void edstop(void);
 
 /*
  * The "quit" command.
  */
 /*ARGSUSED*/
-int
-quitcmd(v)
-	void *v;
+int 
+quitcmd(void *v)
 {
 	/*
 	 * If we are sourcing, then return 1 so execute() can handle it.
@@ -81,71 +83,42 @@ quitcmd(v)
  * Incorporate the any new mail that we found.
  */
 static int
-#ifndef	F_SETLKW
-writeback(res)
-	FILE *res;
-#else
-writeback(res, obuf)
-	FILE *res, *obuf;
-#endif
+writeback(FILE *res, FILE *obuf)
 {
 	struct message *mp;
 	int p, c;
-#ifndef	F_SETLKW
-	FILE *obuf;
-#endif
 
 	p = 0;
-#ifndef	F_SETLKW
-	if ((obuf = Fopen(mailname, "r+")) == (FILE*)NULL) {
-		perror(mailname);
-		return(-1);
-	}
-#else
 	fseek(obuf, 0L, SEEK_SET);
-#endif
 #ifndef APPEND
-	if (res != (FILE*)NULL)
-		while ((c = sgetc(res)) != EOF)
-			(void) sputc(c, obuf);
+	if (res != NULL)
+		while ((c = getc(res)) != EOF)
+			putc(c, obuf);
 #endif
-	for (mp = &message[0]; mp < &message[msgcount]; mp++)
+	for (mp = &message[0]; mp < &message[msgCount]; mp++)
 		if ((mp->m_flag&MPRESERVE)||(mp->m_flag&MTOUCH)==0) {
 			p++;
-			if (send_message(mp, obuf, (struct ignoretab *)0,
-						NULL, CONV_NONE, NULL) < 0) {
+			if (send(mp, obuf, NULL, NULL, SEND_MBOX, NULL) < 0) {
 				perror(mailname);
-#ifndef	F_SETLKW
-				Fclose(obuf);
-#else
 				fseek(obuf, 0L, SEEK_SET);
-#endif
 				return(-1);
 			}
 		}
 #ifdef APPEND
-	if (res != (FILE*)NULL)
-		while ((c = sgetc(res)) != EOF)
-			(void) sputc(c, obuf);
+	if (res != NULL)
+		while ((c = getc(res)) != EOF)
+			putc(c, obuf);
 #endif
 	fflush(obuf);
 	trunc(obuf);
 	if (ferror(obuf)) {
 		perror(mailname);
-#ifndef	F_SETLKW
-		Fclose(obuf);
-#else
 		fseek(obuf, 0L, SEEK_SET);
-#endif
 		return(-1);
 	}
-	if (res != (FILE*)NULL)
+	if (res != NULL)
 		Fclose(res);
-#ifndef	F_SETLKW
-	Fclose(obuf);
-#else
 	fseek(obuf, 0L, SEEK_SET);
-#endif
 	alter(mailname);
 	if (p == 1)
 		printf(catgets(catd, CATSET, 155,
@@ -161,11 +134,11 @@ writeback(res, obuf)
  * Save all untouched messages back in the system mailbox.
  * Remove the system mailbox, if none saved there.
  */
-void
-quit()
+void 
+quit(void)
 {
 	int p, modify, anystat;
-	FILE *fbuf, *rbuf, *readstat = (FILE*)NULL, *abuf;
+	FILE *fbuf, *rbuf, *readstat = NULL, *abuf;
 	struct message *mp;
 	int c;
 	char *tempResid;
@@ -173,16 +146,25 @@ quit()
 
 	/*
 	 * If we are read only, we can't do anything,
-	 * so just return quickly.
+	 * so just return quickly. IMAP can set some
+	 * flags (e.g. "\\Seen") so imap_quit must be
+	 * called even then.
 	 */
-	if (mb.mb_perm == 0)
+	if (mb.mb_perm == 0 && mb.mb_type != MB_IMAP)
 		return;
 	switch (mb.mb_type) {
 	case MB_FILE:
 		break;
+	case MB_MAILDIR:
+		maildir_quit();
+		return;
 	case MB_POP3:
 		pop3_quit();
 		return;
+	case MB_IMAP:
+	case MB_CACHE:
+		imap_quit();
+		return;
 	case MB_VOID:
 		return;
 	}
@@ -205,21 +187,13 @@ quit()
 	 * a message.
 	 */
 
-#ifndef	F_SETLKW
-	fbuf = Fopen(mailname, "r");
-#else
-	fbuf = Fopen(mailname, "r+");
-#endif
-	if (fbuf == (FILE*)NULL) {
+	fbuf = Zopen(mailname, "r+", &mb.mb_compressed);
+	if (fbuf == NULL) {
 		if (errno == ENOENT)
 			return;
 		goto newmail;
 	}
-#ifndef	F_SETLKW
-	if (fcntl_lock(fileno(fbuf), LOCK_EX) == -1) {
-#else
 	if (fcntl_lock(fileno(fbuf), F_WRLCK) == -1) {
-#endif
 nolock:
 		perror(catgets(catd, CATSET, 157, "Unable to lock mailbox"));
 		Fclose(fbuf);
@@ -227,27 +201,27 @@ nolock:
 	}
 	if (dot_lock(mailname, fileno(fbuf), 1, stdout, ".") == -1)
 		goto nolock;
-	rbuf = (FILE *) NULL;
+	rbuf = NULL;
 	if (fstat(fileno(fbuf), &minfo) >= 0 && minfo.st_size > mailsize) {
 		printf(catgets(catd, CATSET, 158, "New mail has arrived.\n"));
 		rbuf = Ftemp(&tempResid, "Rq", "w", 0600, 1);
-		if (rbuf == (FILE*)NULL || fbuf == (FILE*)NULL)
+		if (rbuf == NULL || fbuf == NULL)
 			goto newmail;
 #ifdef APPEND
 		fseek(fbuf, (long)mailsize, SEEK_SET);
-		while ((c = sgetc(fbuf)) != EOF)
-			(void) sputc(c, rbuf);
+		while ((c = getc(fbuf)) != EOF)
+			putc(c, rbuf);
 #else
 		p = minfo.st_size - mailsize;
 		while (p-- > 0) {
-			c = sgetc(fbuf);
+			c = getc(fbuf);
 			if (c == EOF)
 				goto newmail;
-			(void) sputc(c, rbuf);
+			putc(c, rbuf);
 		}
 #endif
 		Fclose(rbuf);
-		if ((rbuf = Fopen(tempResid, "r")) == (FILE*)NULL)
+		if ((rbuf = Fopen(tempResid, "r")) == NULL)
 			goto newmail;
 		rm(tempResid);
 		Ftfree(&tempResid);
@@ -256,10 +230,10 @@ nolock:
 	anystat = holdbits();
 	modify = 0;
 	if (Tflag != NULL) {
-		if ((readstat = Fopen(Tflag, "w")) == (FILE*)NULL)
+		if ((readstat = Zopen(Tflag, "w", NULL)) == NULL)
 			Tflag = NULL;
 	}
-	for (c = 0, p = 0, mp = &message[0]; mp < &message[msgcount]; mp++) {
+	for (c = 0, p = 0, mp = &message[0]; mp < &message[msgCount]; mp++) {
 		if (mp->m_flag & MBOX)
 			c++;
 		if (mp->m_flag & MPRESERVE)
@@ -276,7 +250,7 @@ nolock:
 	}
 	if (readstat != NULL)
 		Fclose(readstat);
-	if (p == msgcount && !modify && !anystat) {
+	if (p == msgCount && !modify && !anystat) {
 		if (p == 1)
 			printf(catgets(catd, CATSET, 155,
 				"Held 1 message in %s\n"), mailname);
@@ -289,11 +263,7 @@ nolock:
 	}
 	if (c == 0) {
 		if (p != 0) {
-#ifndef	F_SETLKW
-			writeback(rbuf);
-#else
 			writeback(rbuf, fbuf);
-#endif
 			Fclose(fbuf);
 			dot_unlock(mailname);
 			return;
@@ -312,11 +282,7 @@ nolock:
 	 */
 
 	if (p != 0) {
-#ifndef	F_SETLKW
-		writeback(rbuf);
-#else
 		writeback(rbuf, fbuf);
-#endif
 		Fclose(fbuf);
 		dot_unlock(mailname);
 		return;
@@ -328,22 +294,13 @@ nolock:
 	 */
 
 cream:
-	if (rbuf != (FILE*)NULL) {
-#ifndef	F_SETLKW
-		abuf = Fopen(mailname, "r+");
-		if (abuf == (FILE*)NULL)
-			goto newmail;
-#else
+	if (rbuf != NULL) {
 		abuf = fbuf;
 		fseek(abuf, 0L, SEEK_SET);
-#endif
-		while ((c = sgetc(rbuf)) != EOF)
-			(void) sputc(c, abuf);
+		while ((c = getc(rbuf)) != EOF)
+			putc(c, abuf);
 		Fclose(rbuf);
 		trunc(abuf);
-#ifndef	F_SETLKW
-		Fclose(abuf);
-#endif
 		alter(mailname);
 		Fclose(fbuf);
 		dot_unlock(mailname);
@@ -356,7 +313,7 @@ cream:
 
 newmail:
 	printf(catgets(catd, CATSET, 166, "Thou hast new mail.\n"));
-	if (fbuf != (FILE*)NULL) {
+	if (fbuf != NULL) {
 		Fclose(fbuf);
 		dot_unlock(mailname);
 	}
@@ -365,8 +322,8 @@ newmail:
 /*
  * Adjust the message flags in each message.
  */
-int
-holdbits()
+int 
+holdbits(void)
 {
 	struct message *mp;
 	int anystat, autohold, holdbit, nohold;
@@ -377,12 +334,13 @@ holdbits()
 	nohold = MBOX|MSAVED|MDELETED|MPRESERVE;
 	if (value("keepsave") != NULL)
 		nohold &= ~MSAVED;
-	for (mp = &message[0]; mp < &message[msgcount]; mp++) {
+	for (mp = &message[0]; mp < &message[msgCount]; mp++) {
 		if (mp->m_flag & MNEW) {
 			mp->m_flag &= ~MNEW;
 			mp->m_flag |= MSTATUS;
 		}
-		if (mp->m_flag & MSTATUS)
+		if (mp->m_flag & (MSTATUS|MFLAG|MUNFLAG|MANSWER|MUNANSWER|
+					MDRAFT|MUNDRAFT))
 			anystat++;
 		if ((mp->m_flag & MTOUCH) == 0)
 			mp->m_flag |= MPRESERVE;
@@ -399,15 +357,16 @@ holdbits()
  * just copy saveable entries at the end.
  */
 
-enum okay
-makembox()
+enum okay 
+makembox(void)
 {
 	struct message *mp;
 	char *mbox, *tempQuit;
 	int mcount, c;
 	FILE *ibuf = NULL, *obuf, *abuf;
+	enum protocol	prot;
 
-	mbox = expand("&");
+	mbox = mboxname;
 	mcount = 0;
 	if (value("append") == NULL) {
 		if ((obuf = Ftemp(&tempQuit, "Rm", "w", 0600, 1)) == NULL) {
@@ -424,9 +383,9 @@ makembox()
 		}
 		rm(tempQuit);
 		Ftfree(&tempQuit);
-		if ((abuf = Fopen(mbox, "r")) != (FILE*)NULL) {
-			while ((c = sgetc(abuf)) != EOF)
-				(void) sputc(c, obuf);
+		if ((abuf = Zopen(mbox, "r", NULL)) != NULL) {
+			while ((c = getc(abuf)) != EOF)
+				putc(c, obuf);
 			Fclose(abuf);
 		}
 		if (ferror(obuf)) {
@@ -438,30 +397,38 @@ makembox()
 		}
 		Fclose(obuf);
 		close(creat(mbox, 0600));
-		if ((obuf = Fopen(mbox, "r+")) == (FILE*)NULL) {
+		if ((obuf = Zopen(mbox, "r+", NULL)) == NULL) {
 			perror(mbox);
 			Fclose(ibuf);
 			return STOP;
 		}
 	}
 	else {
-		if ((obuf = Fopen(mbox, "a")) == (FILE*)NULL) {
+		if ((obuf = Zopen(mbox, "a", NULL)) == NULL) {
 			perror(mbox);
 			return STOP;
 		}
 		fchmod(fileno(obuf), 0600);
 	}
-	for (mp = &message[0]; mp < &message[msgcount]; mp++)
+	prot = which_protocol(mbox);
+	for (mp = &message[0]; mp < &message[msgCount]; mp++)
 		if (mp->m_flag & MBOX) {
 			mcount++;
-			if (send_message(mp, obuf, saveignore,
-						NULL, CONV_NONE, NULL) < 0) {
+			if (prot == PROTO_IMAP &&
+					saveignore[0].i_count == 0 &&
+					saveignore[1].i_count == 0 &&
+					imap_thisaccount(mbox)) {
+				if (imap_copy(mp, mp-message+1, mbox) == STOP)
+					goto err;
+			} else if (send(mp, obuf, saveignore,
+						NULL, SEND_MBOX, NULL) < 0) {
 				perror(mbox);
-				if (ibuf)
+			err:	if (ibuf)
 					Fclose(ibuf);
 				Fclose(obuf);
 				return STOP;
 			}
+			mp->m_flag |= MBOXED;
 		}
 
 	/*
@@ -472,12 +439,12 @@ makembox()
 
 	if (value("append") == NULL) {
 		rewind(ibuf);
-		c = sgetc(ibuf);
+		c = getc(ibuf);
 		while (c != EOF) {
-			(void) sputc(c, obuf);
+			putc(c, obuf);
 			if (ferror(obuf))
 				break;
-			c = sgetc(ibuf);
+			c = getc(ibuf);
 		}
 		Fclose(ibuf);
 		fflush(obuf);
@@ -488,7 +455,11 @@ makembox()
 		Fclose(obuf);
 		return STOP;
 	}
-	Fclose(obuf);
+	if (Fclose(obuf) != 0) {
+		if (prot != PROTO_IMAP)
+			perror(mbox);
+		return STOP;
+	}
 	if (mcount == 1)
 		printf(catgets(catd, CATSET, 164, "Saved 1 message in mbox\n"));
 	else
@@ -501,27 +472,28 @@ makembox()
  * Terminate an editing session by attempting to write out the user's
  * file from the temporary.  Save any new stuff appended to the file.
  */
-static void
-edstop()
+static void 
+edstop(void)
 {
 	int gotcha, c;
 	struct message *mp;
-	FILE *obuf, *ibuf = NULL, *readstat = (FILE*)NULL;
+	FILE *obuf, *ibuf = NULL, *readstat = NULL;
 	struct stat statb;
 
 	if (mb.mb_perm == 0)
 		return;
 	holdsigs();
 	if (Tflag != NULL) {
-		if ((readstat = Fopen(Tflag, "w")) == (FILE*)NULL)
+		if ((readstat = Zopen(Tflag, "w", NULL)) == NULL)
 			Tflag = NULL;
 	}
-	for (mp = &message[0], gotcha = 0; mp < &message[msgcount]; mp++) {
+	for (mp = &message[0], gotcha = 0; mp < &message[msgCount]; mp++) {
 		if (mp->m_flag & MNEW) {
 			mp->m_flag &= ~MNEW;
 			mp->m_flag |= MSTATUS;
 		}
-		if (mp->m_flag & (MODIFY|MDELETED|MSTATUS))
+		if (mp->m_flag & (MODIFY|MDELETED|MSTATUS|MFLAG|MUNFLAG|
+					MANSWER|MUNANSWER|MDRAFT|MUNDRAFT))
 			gotcha++;
 		if (readstat != NULL && (mp->m_flag & (MREAD|MDELETED)) != 0) {
 			char *id;
@@ -535,7 +507,7 @@ edstop()
 		Fclose(readstat);
 	if (!gotcha || Tflag != NULL)
 		goto done;
-	ibuf = (FILE *)NULL;
+	ibuf = NULL;
 	if (stat(mailname, &statb) >= 0 && statb.st_size > mailsize) {
 		char *tempname;
 
@@ -544,7 +516,7 @@ edstop()
 			relsesigs();
 			reset(0);
 		}
-		if ((ibuf = Fopen(mailname, "r")) == (FILE *)NULL) {
+		if ((ibuf = Zopen(mailname, "r", &mb.mb_compressed)) == NULL) {
 			perror(mailname);
 			Fclose(obuf);
 			rm(tempname);
@@ -553,11 +525,11 @@ edstop()
 			reset(0);
 		}
 		fseek(ibuf, (long)mailsize, SEEK_SET);
-		while ((c = sgetc(ibuf)) != EOF)
-			(void) sputc(c, obuf);
+		while ((c = getc(ibuf)) != EOF)
+			putc(c, obuf);
 		Fclose(ibuf);
 		Fclose(obuf);
-		if ((ibuf = Fopen(tempname, "r")) == (FILE*)NULL) {
+		if ((ibuf = Fopen(tempname, "r")) == NULL) {
 			perror(tempname);
 			rm(tempname);
 			Ftfree(&tempname);
@@ -569,28 +541,27 @@ edstop()
 	}
 	printf(catgets(catd, CATSET, 168, "\"%s\" "), mailname);
 	fflush(stdout);
-	if ((obuf = Fopen(mailname, "r+")) == (FILE*)NULL) {
+	if ((obuf = Zopen(mailname, "r+", &mb.mb_compressed)) == NULL) {
 		perror(mailname);
 		relsesigs();
 		reset(0);
 	}
 	trunc(obuf);
 	c = 0;
-	for (mp = &message[0]; mp < &message[msgcount]; mp++) {
+	for (mp = &message[0]; mp < &message[msgCount]; mp++) {
 		if ((mp->m_flag & MDELETED) != 0)
 			continue;
 		c++;
-		if (send_message(mp, obuf, (struct ignoretab *) NULL,
-					NULL, CONV_NONE, NULL) < 0) {
+		if (send(mp, obuf, NULL, NULL, SEND_MBOX, NULL) < 0) {
 			perror(mailname);
 			relsesigs();
 			reset(0);
 		}
 	}
-	gotcha = (c == 0 && ibuf == (FILE*)NULL);
-	if (ibuf != (FILE*)NULL) {
-		while ((c = sgetc(ibuf)) != EOF)
-			(void) sputc(c, obuf);
+	gotcha = (c == 0 && ibuf == NULL);
+	if (ibuf != NULL) {
+		while ((c = getc(ibuf)) != EOF)
+			putc(c, obuf);
 		Fclose(ibuf);
 	}
 	fflush(obuf);
